@@ -66,9 +66,11 @@ use crate::channel::reactor::{BoxedChannelSink, BoxedChannelStream, CtrlMsg, Rea
 pub use crate::channel::unique_id::UniqId;
 use crate::circuit;
 use crate::circuit::celltypes::CreateResponse;
+use crate::util::ts::OptTimestamp;
 use crate::{Error, Result};
 use std::pin::Pin;
 use tor_cell::chancell::{msg, ChanCell, CircId};
+use tor_error::internal;
 use tor_linkspec::ChanTarget;
 use tor_llcrypto::pk::ed25519::Ed25519Identity;
 use tor_llcrypto::pk::rsa::RsaIdentity;
@@ -86,6 +88,8 @@ use tracing::trace;
 
 // reexport
 use crate::channel::unique_id::CircUniqIdContext;
+#[cfg(test)]
+pub(crate) use codec::CodecError;
 pub use handshake::{OutboundClientHandshake, UnverifiedChannel, VerifiedChannel};
 
 /// Type alias: A Sink and Stream that transforms a TLS connection into
@@ -128,6 +132,11 @@ pub(crate) struct ChannelDetails {
     rsa_id: RsaIdentity,
     /// If true, this channel is closing.
     closed: AtomicBool,
+    /// Since when the channel became unused.
+    ///
+    /// If calling `time_since_update` returns None,
+    /// this channel is still in use by at least one circuit.
+    unused_since: OptTimestamp,
 }
 
 impl Sink<ChanCell> for Channel {
@@ -245,12 +254,15 @@ impl Channel {
         let (control_tx, control_rx) = mpsc::unbounded();
         let (cell_tx, cell_rx) = mpsc::channel(CHANNEL_BUFFER_SIZE);
         let closed = AtomicBool::new(false);
+        let unused_since = OptTimestamp::new();
+        unused_since.update();
 
         let details = ChannelDetails {
             unique_id,
             ed25519_id,
             rsa_id,
             closed,
+            unused_since,
         };
         let details = Arc::new(details);
 
@@ -316,18 +328,29 @@ impl Channel {
         self.details.closed.load(Ordering::SeqCst)
     }
 
+    /// If the channel is not in use, return the amount of time
+    /// it has had with no circuits.
+    ///
+    /// Return `None` if the channel is currently in use.
+    pub fn duration_unused(&self) -> Option<std::time::Duration> {
+        self.details
+            .unused_since
+            .time_since_update()
+            .map(Into::into)
+    }
+
     /// Check whether a cell type is permissible to be _sent_ on an
     /// open client channel.
     fn check_cell(&self, cell: &ChanCell) -> Result<()> {
         use msg::ChanMsg::*;
         let msg = cell.msg();
         match msg {
-            Created(_) | Created2(_) | CreatedFast(_) => Err(Error::InternalError(format!(
+            Created(_) | Created2(_) | CreatedFast(_) => Err(Error::from(internal!(
                 "Can't send {} cell on client channel",
                 msg.cmd()
             ))),
             Certs(_) | Versions(_) | Authenticate(_) | Authorize(_) | AuthChallenge(_)
-            | Netinfo(_) => Err(Error::InternalError(format!(
+            | Netinfo(_) => Err(Error::from(internal!(
                 "Can't send {} cell after handshake is done",
                 msg.cmd()
             ))),
@@ -423,16 +446,7 @@ pub(crate) mod test {
     use tor_cell::chancell::{msg, ChanCell};
 
     /// Make a new fake reactor-less channel.  For testing only, obviously.
-    pub(crate) fn fake_channel() -> Channel {
-        let unique_id = UniqId::new();
-
-        let details = Arc::new(ChannelDetails {
-            unique_id,
-            ed25519_id: [6_u8; 32].into(),
-            rsa_id: [10_u8; 20].into(),
-            closed: AtomicBool::new(false),
-        });
-
+    pub(crate) fn fake_channel(details: Arc<ChannelDetails>) -> Channel {
         Channel {
             control: mpsc::unbounded().0,
             cell_tx: mpsc::channel(CHANNEL_BUFFER_SIZE).0,
@@ -440,25 +454,34 @@ pub(crate) mod test {
         }
     }
 
+    fn fake_channel_details() -> Arc<ChannelDetails> {
+        let unique_id = UniqId::new();
+        let unused_since = OptTimestamp::new();
+
+        Arc::new(ChannelDetails {
+            unique_id,
+            ed25519_id: [6_u8; 32].into(),
+            rsa_id: [10_u8; 20].into(),
+            closed: AtomicBool::new(false),
+            unused_since,
+        })
+    }
+
     #[test]
     fn send_bad() {
         tor_rtcompat::test_with_all_runtimes!(|_rt| async move {
-            let chan = fake_channel();
+            let chan = fake_channel(fake_channel_details());
 
             let cell = ChanCell::new(7.into(), msg::Created2::new(&b"hihi"[..]).into());
             let e = chan.check_cell(&cell);
             assert!(e.is_err());
-            assert_eq!(
-                format!("{}", e.unwrap_err()),
-                "Internal programming error: Can't send CREATED2 cell on client channel"
-            );
+            assert!(format!("{}", e.unwrap_err())
+                .contains("Can't send CREATED2 cell on client channel"));
             let cell = ChanCell::new(0.into(), msg::Certs::new_empty().into());
             let e = chan.check_cell(&cell);
             assert!(e.is_err());
-            assert_eq!(
-                format!("{}", e.unwrap_err()),
-                "Internal programming error: Can't send CERTS cell after handshake is done"
-            );
+            assert!(format!("{}", e.unwrap_err())
+                .contains("Can't send CERTS cell after handshake is done"));
 
             let cell = ChanCell::new(5.into(), msg::Create2::new(2, &b"abc"[..]).into());
             let e = chan.check_cell(&cell);
@@ -480,7 +503,7 @@ pub(crate) mod test {
     #[test]
     fn check_match() {
         use std::net::SocketAddr;
-        let chan = fake_channel();
+        let chan = fake_channel(fake_channel_details());
 
         struct ChanT {
             ed_id: Ed25519Identity,
@@ -519,8 +542,16 @@ pub(crate) mod test {
 
     #[test]
     fn unique_id() {
-        let ch1 = fake_channel();
-        let ch2 = fake_channel();
+        let ch1 = fake_channel(fake_channel_details());
+        let ch2 = fake_channel(fake_channel_details());
         assert_ne!(ch1.unique_id(), ch2.unique_id());
     }
+
+    #[test]
+    fn duration_unused_at() {
+        let details = fake_channel_details();
+        let ch = fake_channel(Arc::clone(&details));
+        details.unused_since.update();
+        assert!(ch.duration_unused().is_some());
+    }
 }
