@@ -15,12 +15,14 @@ use tokio_crate::io::{AsyncRead as TokioAsyncRead, AsyncWrite as TokioAsyncWrite
 #[cfg(feature = "tokio")]
 use tokio_util::compat::{FuturesAsyncReadCompatExt, FuturesAsyncWriteCompatExt};
 
+use std::fmt::{self, Debug};
 use std::io::Result as IoResult;
 use std::pin::Pin;
 
 use crate::circuit::StreamTarget;
 use crate::stream::StreamReader;
 use tor_cell::relaycell::msg::{Data, RelayMsg};
+use tor_error::internal;
 
 /// An anonymized stream over the Tor network.
 ///
@@ -85,6 +87,7 @@ use tor_cell::relaycell::msg::{Data, RelayMsg};
 // Note that this type is re-exported as a part of the public API of
 // the `arti-client` crate.  Any changes to its API here in
 // `tor-proto` need to be reflected above.
+#[derive(Debug)]
 pub struct DataStream {
     /// Underlying writer for this stream
     w: DataWriter,
@@ -108,6 +111,7 @@ pub struct DataStream {
 // Note that this type is re-exported as a part of the public API of
 // the `arti-client` crate.  Any changes to its API here in
 // `tor-proto` need to be reflected above.
+#[derive(Debug)]
 pub struct DataWriter {
     /// Internal state for this writer
     ///
@@ -132,6 +136,7 @@ pub struct DataWriter {
 // Note that this type is re-exported as a part of the public API of
 // the `arti-client` crate.  Any changes to its API here in
 // `tor-proto` need to be reflected above.
+#[derive(Debug)]
 pub struct DataReader {
     /// Internal state for this reader.
     ///
@@ -188,9 +193,10 @@ impl DataStream {
             self.r.state = Some(DataReaderState::Ready(imp));
             result
         } else {
-            Err(Error::InternalError(
-                "Expected ready state of a new stream.".to_owned(),
-            ))
+            Err(Error::from(internal!(
+                "Expected ready state, got {:?}",
+                state
+            )))
         }
     }
 }
@@ -262,6 +268,16 @@ enum DataWriterState {
     Flushing(Pin<Box<dyn Future<Output = (DataWriterImpl, Result<()>)> + Send>>),
 }
 
+impl Debug for DataWriterState {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            DataWriterState::Closed => write!(f, "DataWriterState::Closed"),
+            DataWriterState::Ready(i) => write!(f, "DataWriterState::Ready({:?})", i),
+            DataWriterState::Flushing(_) => write!(f, "DataWriterState::Flushing(..)"),
+        }
+    }
+}
+
 /// Internal: the write part of a DataStream
 struct DataWriterImpl {
     /// The underlying StreamTarget object.
@@ -278,6 +294,15 @@ struct DataWriterImpl {
     n_pending: usize,
 }
 
+impl Debug for DataWriterImpl {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_struct("DataWriterImpl")
+            .field("s", &self.s)
+            .field("n_pending", &self.n_pending)
+            .finish_non_exhaustive()
+    }
+}
+
 impl DataWriter {
     /// Helper for poll_flush() and poll_close(): Performs a flush, then
     /// closes the stream if should_close is true.
@@ -451,6 +476,16 @@ enum DataReaderState {
     ReadingCell(Pin<Box<dyn Future<Output = (DataReaderImpl, Result<()>)> + Send>>),
 }
 
+impl std::fmt::Debug for DataReaderState {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Self::Closed => write!(f, "Closed"),
+            Self::Ready(_) => write!(f, "Ready"),
+            Self::ReadingCell(_) => write!(f, "ReadingCell"),
+        }
+    }
+}
+
 /// Wrapper for the read part of a DataStream
 struct DataReaderImpl {
     /// The underlying StreamReader object.
